﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;
using System.Xml;

namespace WP7Push_Notifier.Tools
{
    public class NotificationSender
    {
        public const int MAX_PAYLOAD_LENGTH = 1024;

        public void SendRawNotification(List<Uri> Uris, byte[] Payload)
        {
            foreach (var uri in Uris)
                SendMessage(uri, Payload, NotificationType.Raw);
        }

        public void SendToastNotification(List<Uri> Uris, string message1, string message2)
        {
            byte[] payload = PrepareToastPayload(message1, message2);

            foreach (var uri in Uris)
                SendMessage(uri, payload, NotificationType.Toast);
        }

        public void SendTileNotification(List<Uri> Uris, string BackgroundImageUri, int Count, string Title)
        {
            byte[] payload = PrepareTilePayload(BackgroundImageUri, Count, Title);

            foreach (var uri in Uris)
                SendMessage(uri, payload, NotificationType.Tile);
        }

        private void SendMessage(Uri channelUri, byte[] payload, NotificationType notificationType)
        {
            //Check the length of the payload and reject it if too long
            if (payload.Length > MAX_PAYLOAD_LENGTH)
                throw new ArgumentOutOfRangeException("Payload is too long. Maximum payload size shouldn't exceed " + MAX_PAYLOAD_LENGTH.ToString() + " bytes");

            try
            {
                //prepare request
                HttpWebRequest request = (HttpWebRequest)WebRequest.Create(channelUri);
                request.Method = WebRequestMethods.Http.Post;
                request.ContentType = "text/xml; charset=utf-8";
                request.ContentLength = payload.Length;
                request.Headers.Add("X-MessageID", Guid.NewGuid().ToString());
                request.Headers["X-NotificationClass"] = ((int)notificationType).ToString();

                if (notificationType == NotificationType.Toast)
                    request.Headers["X-WindowsPhone-Target"] = "toast";
                else if (notificationType == NotificationType.Tile)
                    request.Headers["X-WindowsPhone-Target"] = "token";

                using (Stream requestStream = request.GetRequestStream())
                {
                    requestStream.Write(payload, 0, payload.Length);
                    requestStream.Close();
                }

                // Sends the notification and gets the response.
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                string notificationStatus = response.Headers["X-NotificationStatus"];
                string notificationChannelStatus = response.Headers["X-SubscriptionStatus"];
                string deviceConnectionStatus = response.Headers["X-DeviceConnectionStatus"];
                System.Diagnostics.Debug.WriteLine("NotificationStatus: {0}\nChannelStatus: {1}\nDeviceConnectionStatus: {2}", notificationStatus, notificationChannelStatus, deviceConnectionStatus);

            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.ToString());
            }
        }

        #region Prepare Payloads
        //##############################################################
        //http://msdn.microsoft.com/en-us/library/ff402545(v=VS.92).aspx
        //##############################################################

        private static byte[] PrepareToastPayload(string text1, string text2)
        {
            string toastSkeleton = "<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
                "<wp:Notification xmlns:wp=\"WPNotification\">" + 
                    "<wp:Toast>" +
                        "<wp:Text1>{0}</wp:Text1>" +
                        "<wp:Text2>{1}</wp:Text2>" +
                    "</wp:Toast>" +
                "</wp:Notification>";

            return Encoding.UTF8.GetBytes(String.Format(toastSkeleton, text1, text2));
        }

        private static byte[] PrepareTilePayload(string backgroundImageUri, int count, string title)
        {
            string tileSkeleton = "<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
                    "<wp:Notification xmlns:wp=\"WPNotification\">" +
                        "<wp:Tile>" +
                            "<wp:BackgroundImage>{0}</wp:BackgroundImage>" +
                            "<wp:Count>{1}</wp:Count>" +
                            "<wp:Title>{2}</wp:Title>" +
                        "</wp:Tile>" +
                    "</wp:Notification>";

            return Encoding.UTF8.GetBytes(String.Format(tileSkeleton, backgroundImageUri, count, title));
        }
        
        #endregion
    }
}
